--CREATE TABLE IF NOT EXISTS "Task" (
--	"id_task"   INTEGER PRIMARY KEY AUTOINCREMENT,
--	"type" TEXT,
--	"step" INTEGER,
--	"retry_count"   INTEGER NOT NULL,
--	"media_id"  INTEGER,
--	"file_id"   INTEGER,
--	"mrl" TEXT,
--	"path" TEXT,
--	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON DELETE CASCADE,
--	FOREIGN KEY("file_id") REFERENCES "File"("id_file") ON DELETE CASCADE
--);

create TABLE IF NOT EXISTS "Chapter" (
	"id_chapter"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"offset"	INTEGER NOT NULL,
	"duration"	INTEGER NOT NULL,
	"name"	TEXT,
	"media_id"	INTEGER,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);


create TABLE IF NOT EXISTS "SubtitleTrack" (
	"id_track"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"codec"	TEXT,
	"language"	TEXT,
	"description"	TEXT,
	"encoding"	TEXT,
	"media_id"	UNSIGNED INT,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);
create TABLE IF NOT EXISTS "Metadata" (
	"id_media"	INTEGER,
	"entity_type"	INTEGER,
	"type"	INTEGER,
	"value"	TEXT,
	PRIMARY KEY("id_media","entity_type","type")
);

create TABLE IF NOT EXISTS "Settings" (
	"db_model_version"	UNSIGNED INTEGER NOT NULL
);

create TABLE IF NOT EXISTS "MediaArtistRelation" (
	"media_id"	INTEGER NOT NULL,
	"artist_id"	INTEGER,
	FOREIGN KEY("artist_id") REFERENCES "Artist"("id_artist") ON delete CASCADE,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE,
	PRIMARY KEY("media_id","artist_id")
);

create TABLE IF NOT EXISTS "Artist" (
	"id_artist"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"name"	TEXT UNIQUE COLLATE NOCASE,
	"shortbio"	TEXT,
	"thumbnail_id"	INTEGER,
	"nb_albums"	UNSIGNED INT DEFAULT 0,
	"nb_tracks"	UNSIGNED INT DEFAULT 0,
	"mb_id"	TEXT,
	"is_present"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	FOREIGN KEY("thumbnail_id") REFERENCES "Thumbnail"("id_thumbnail")
);

create TABLE IF NOT EXISTS "AudioTrack" (
	"id_track"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"codec"	TEXT,
	"bitrate"	UNSIGNED INTEGER,
	"samplerate"	UNSIGNED INTEGER,
	"nb_channels"	UNSIGNED INTEGER,
	"language"	TEXT,
	"description"	TEXT,
	"media_id"	UNSIGNED INT,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);

create TABLE IF NOT EXISTS "VideoTrack" (
	"id_track"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"codec"	TEXT,
	"width"	UNSIGNED INTEGER,
	"height"	UNSIGNED INTEGER,
	"fps_num"	UNSIGNED INTEGER,
	"fps_den"	UNSIGNED INTEGER,
	"bitrate"	UNSIGNED INTEGER,
	"sar_num"	UNSIGNED INTEGER,
	"sar_den"	UNSIGNED INTEGER,
	"media_id"	UNSIGNED INT,
	"orientation" UNSIGNED INTEGER,
	"language"	TEXT,
	"description"	TEXT,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);
create TABLE IF NOT EXISTS "Movie" (
	"id_movie"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"media_id"	UNSIGNED INTEGER NOT NULL,
	"summary"	TEXT,
	"imdb_id"	TEXT,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);
create TABLE IF NOT EXISTS "ShowEpisode" (
	"id_episode"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"media_id"	UNSIGNED INTEGER NOT NULL,
	"episode_number"	UNSIGNED INT,
	"season_number"	UNSIGNED INT,
	"episode_summary"	TEXT,
	"tvdb_id"	TEXT,
	"show_id"	UNSIGNED INT,
	FOREIGN KEY("show_id") REFERENCES "Show"("id_show") ON delete CASCADE,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE
);

create TABLE IF NOT EXISTS "Show" (
	"id_show"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"title"	TEXT,
	"release_date"	UNSIGNED INTEGER,
	"short_summary"	TEXT,
	"artwork_mrl"	TEXT,
	"tvdb_id"	TEXT
);
create TABLE IF NOT EXISTS "AlbumTrack" (
	"id_track"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"media_id"	INTEGER UNIQUE,
	"duration"	INTEGER NOT NULL,
	"artist_id"	UNSIGNED INTEGER,
	"genre_id"	INTEGER,
	"track_number"	UNSIGNED INTEGER,
	"album_id"	UNSIGNED INTEGER NOT NULL,
	"disc_number"	UNSIGNED INTEGER,
	FOREIGN KEY("album_id") REFERENCES "Album"("id_album") ON delete CASCADE,
	FOREIGN KEY("artist_id") REFERENCES "Artist"("id_artist") ON delete CASCADE,
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE,
	FOREIGN KEY("genre_id") REFERENCES "Genre"("id_genre")
);

create TABLE IF NOT EXISTS "Album" (
	"id_album"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"title"	TEXT COLLATE NOCASE,
	"artist_id"	UNSIGNED INTEGER,
	"release_year"	UNSIGNED INTEGER,
	"short_summary"	TEXT,
	"thumbnail_id"	UNSIGNED INT,
	"nb_tracks"	UNSIGNED INTEGER DEFAULT 0,
	"duration"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	"nb_discs"	UNSIGNED INTEGER NOT NULL DEFAULT 1,
	"is_present"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	FOREIGN KEY("thumbnail_id") REFERENCES "Thumbnail"("id_thumbnail"),
	FOREIGN KEY("artist_id") REFERENCES "Artist"("id_artist") ON delete CASCADE
);

create TABLE IF NOT EXISTS "Genre" (
	"id_genre"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"name"	TEXT UNIQUE COLLATE NOCASE,
	"nb_tracks"	INTEGER NOT NULL DEFAULT 0
);

create TABLE IF NOT EXISTS "PlaylistMediaRelation" (
	"file_id"	INTEGER,
	"mrl"	STRING,
	"playlist_id"	INTEGER,
	"position"	INTEGER,
	"tmp_del"	BOOLEAN NOT NULL DEFAULT 0,
	FOREIGN KEY("file_id") REFERENCES "File"("id_file") ON delete SET NULL,
	FOREIGN KEY("playlist_id") REFERENCES "Playlist"("id_playlist") ON delete CASCADE
);
create TABLE IF NOT EXISTS "Playlist" (
	"id_playlist"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"name"	TEXT COLLATE NOCASE,
	"file_id"	UNSIGNED INT DEFAULT NULL,
	"creation_date"	UNSIGNED INT NOT NULL,
	"artwork_mrl"	TEXT,
	FOREIGN KEY("file_id") REFERENCES "File"("id_file") ON delete CASCADE
);

create TABLE IF NOT EXISTS "LabelFileRelation" (
	"label_id"	INTEGER,
	"media_id"	INTEGER,
	PRIMARY KEY("label_id","media_id"),
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE,
	FOREIGN KEY("label_id") REFERENCES "Label"("id_label") ON delete CASCADE
);
create TABLE IF NOT EXISTS "Label" (
	"id_label"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"name"	TEXT UNIQUE
);

-- path 는 실제 파일이 있는 경로, mrl은 가상 경로이거나 실제파일이 있는 경로
create TABLE IF NOT EXISTS "File" (
	"id_file"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"media_id"	UNSIGNED INT DEFAULT NULL,
	"playlist_id"	UNSIGNED INT DEFAULT NULL,
	"mrl"	TEXT,
	"path"  TEXT,
	"type"	UNSIGNED INTEGER,
	"last_modification_date"	UNSIGNED INT,
	"size"	UNSIGNED INT,
	"folder_id"	UNSIGNED INT,
	"device_id" UNSIGNED INT,
	"meta_fail_cnt" UNSIGNED INT DEFAULT 0,
	"is_removable"	BOOLEAN NOT NULL DEFAULT 0,
	"is_external"	BOOLEAN NOT NULL DEFAULT 0,
	"is_network"	BOOLEAN NOT NULL DEFAULT 0,
	"is_hidden"     BOOLEAN  NOT NULL DEFAULT 0,
	"working_key"    TEXT NULL,
	"is_working"    BOOLEAN NOT NULL DEFAULT 0,
	FOREIGN KEY("playlist_id") REFERENCES "Playlist"("id_playlist") ON delete CASCADE,
	UNIQUE("mrl","folder_id"),
	FOREIGN KEY("media_id") REFERENCES "Media"("id_media") ON delete CASCADE,
	FOREIGN KEY("folder_id") REFERENCES "Folder"("id_folder") ON delete CASCADE
);

create TABLE IF NOT EXISTS "Media" (
	"id_media"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"type"	INTEGER,
	"subtype"	INTEGER NOT NULL DEFAULT 0,
	"duration"	INTEGER DEFAULT -1,
	"play_count"	UNSIGNED INTEGER,
	"last_played_date"	UNSIGNED INTEGER,
	"real_last_played_date"	UNSIGNED INTEGER,
	"insertion_date"	UNSIGNED INTEGER,
	"release_date"	UNSIGNED INTEGER,
	"thumbnail_id"	INTEGER,
	"thumbnail_fail_cnt" UNSIGNED INTEGER DEFAULT 0,
	"title"	TEXT COLLATE NOCASE,
	"filename"	TEXT COLLATE NOCASE,
	"is_favorite"	BOOLEAN NOT NULL DEFAULT 0,
	"is_present"	BOOLEAN NOT NULL DEFAULT 1,
	"device_id"	INTEGER,
	"nb_playlists"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	"folder_id"	UNSIGNED INTEGER,
	"is_hidden" BOOLEAN  NOT NULL DEFAULT 0,
	"file_id"	UNSIGNED INTEGER NOT NULL,
	"is_visible" BOOLEAN  NOT NULL DEFAULT 1,
	"is_lock" BOOLEAN  NOT NULL DEFAULT 0,
	"is_like" BOOLEAN  NOT NULL DEFAULT 0,

	FOREIGN KEY("folder_id") REFERENCES "Folder"("id_folder"),
	FOREIGN KEY("thumbnail_id") REFERENCES "Thumbnail"("id_thumbnail")
);
create TABLE IF NOT EXISTS "Thumbnail" (
	"id_thumbnail"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"mrl"	TEXT,
	"origin"	INTEGER NOT NULL,
	"is_generated"	BOOLEAN NOT NULL
);

create TABLE IF NOT EXISTS "ExcludedEntryFolder" (
	"folder_id"	UNSIGNED INTEGER NOT NULL UNIQUE,
	FOREIGN KEY("folder_id") REFERENCES "Folder"("id_folder") ON delete CASCADE
);


create TABLE IF NOT EXISTS "Folder" (
	"id_folder"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"path"	TEXT,
	"name"	TEXT COLLATE NOCASE,
	"parent_id"	UNSIGNED INTEGER,
	"is_banned"	BOOLEAN NOT NULL DEFAULT 0,
	"device_id"	UNSIGNED INTEGER,
	"is_removable"	BOOLEAN NOT NULL DEFAULT 0,
	"nb_audio"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	"nb_video"	UNSIGNED INTEGER NOT NULL DEFAULT 0,
	"is_favorite" BOOLEAN NOT NULL DEFAULT 0,
	"color" UNSIGNED INTEGER,
    "is_hidden" BOOLEAN  NOT NULL DEFAULT 0,
    "is_root" BOOLEAN NOT NULL DEFAULT 0,
    "signin_required" BOOLEAN NOT NULL DEFAULT 0,
	"user_id"       TEXT NULL,
	"password"      TEXT NULL,
    "is_visible" BOOLEAN  NOT NULL DEFAULT 1,
    "is_lock" BOOLEAN  NOT NULL DEFAULT 0,


	UNIQUE("path","device_id"),
	FOREIGN KEY("device_id") REFERENCES "Device"("id_device") ON delete CASCADE,
	FOREIGN KEY("parent_id") REFERENCES "Folder"("id_folder") ON delete CASCADE
);


-- 디바이스 테이블
-- "is_present"	사용할 수 있는 상태면 1 (true)
-- "signin_required" 로그인이 필요한 경우 1 (true)
-- "network_auto_discovered" 네트워크 검색 자동으로 추가되면 1 (true), 유저가 등록한 네트워크면 0
--	"port"          INTEGER NOT NULL DEFAULT -1,
create TABLE IF NOT EXISTS "Device" (
	"id_device"	INTEGER PRIMARY KEY AUTOINCREMENT,
	"uuid"	TEXT UNIQUE COLLATE NOCASE,
	"scheme"	TEXT,
	"is_removable"	BOOLEAN,
	"is_present"	BOOLEAN,
	"is_network"    BOOLEAN NOT NULL DEFAULT 0,
	"signin_required" BOOLEAN NOT NULL DEFAULT 0,
	"network_auto_discovered" BOOLEAN NOT NULL DEFAULT 0,
	"user_id"       TEXT NULL,
	"password"      TEXT NULL,
	"home"          TEXT NULL,
	"server"        TEXT NULL,
	"artwork"       TEXT NULL,
	"title"      TEXT NULL,

	"last_seen"	UNSIGNED INTEGER
);

-- 클라우드 정보 저장 테이블
-- "id" 내부적으로 사용하는 unique id
-- "cid" 클라우드 회사마다 넘겨주는 고유 id, google:email, dropBox:email
create TABLE IF NOT EXISTS "Cloud" (
	"id"	TEXT PRIMARY KEY COLLATE NOCASE,
	"name"	TEXT,
	"type"	UNSIGNED INTEGER,
	"cid"	TEXT,
	"insertion_date" UNSIGNED INTEGER
);


create INDEX IF NOT EXISTS "folder_parent_id" ON "Folder" (
	"parent_id"
);

create INDEX IF NOT EXISTS "folder_device_id" ON "Folder" (
	"device_id"
);

create INDEX IF NOT EXISTS "folder_path_idx" ON "Folder" (
	"path"
);

create INDEX IF NOT EXISTS "file_folder_id_index" ON "File" (
	"folder_id"
);

create INDEX IF NOT EXISTS "file_media_id_index" ON "File" (
	"media_id"
);

create INDEX IF NOT EXISTS "file_mrl_idx" ON "File" (
	"mrl"
);

create INDEX IF NOT EXISTS "index_media_presence" ON "Media" (
	"is_present"
);
create INDEX IF NOT EXISTS "index_last_played_date" ON "Media" (
	"last_played_date"	DESC
);
create INDEX IF NOT EXISTS "media_types_idx" ON "Media" (
	"type",
	"subtype"
);


create INDEX IF NOT EXISTS "album_media_artist_genre_album_idx" ON "AlbumTrack" (
	"media_id",
	"artist_id",
	"genre_id",
	"album_id"
);
create INDEX IF NOT EXISTS "album_artist_id_idx" ON "Album" (
	"artist_id"
);

create INDEX IF NOT EXISTS "album_title_releaseyear_idx" ON "Album" (
	"title",
	"release_year"
);


create INDEX IF NOT EXISTS "parent_folder_id_idx" ON "Folder" (
	"parent_id"
);
create INDEX IF NOT EXISTS "folder_device_id_idx" ON "Folder" (
	"device_id"
);
create INDEX IF NOT EXISTS "subtitle_track_media_idx" ON "SubtitleTrack" (
	"media_id"
);
create INDEX IF NOT EXISTS "audio_track_media_idx" ON "AudioTrack" (
	"media_id"
);
create INDEX IF NOT EXISTS "video_track_media_idx" ON "VideoTrack" (
	"media_id"
);
create INDEX IF NOT EXISTS "movie_media_idx" ON "Movie" (
	"media_id"
);
create INDEX IF NOT EXISTS "show_episode_media_show_idx" ON "ShowEpisode" (
	"media_id",
	"show_id"
);
create INDEX IF NOT EXISTS "album_track_album_genre_artist_ids" ON "AlbumTrack" (
	"album_id",
	"genre_id",
	"artist_id"
);
create INDEX IF NOT EXISTS "playlist_file_pl_id_index" ON "PlaylistMediaRelation" (
	"file_id",
	"playlist_id"
);

-- 재생대기열을 사용하기 위한 강제적인 값.
insert into "Playlist" ("id_playlist", "name", "creation_date") values (-1, "system", "0");